/*
Dozen Days of Dice episode 12: Dix Mille
Copyright (c) 2008 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.dozendays.dice.dixmille
{
	import flash.display.SimpleButton;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;

	import com.blazinggames.dozendays.*;
	import com.blazinggames.dozendays.dice.*;

	public class DixMilleGame
	extends Sprite
	{
// ***** CONSTANTS **********************************************************

		public static const ROLL_TEXT:String = "Roll!";
		public static const DONE_TEXT:String = "Done!";
		public static const GROUP_TEXT:String = "Group";
		public static const SCORE_FORMAT:TextFormat = new TextFormat(null, 18, 0x0FFFFFF, null, null, null, null, null, TextFormatAlign.CENTER);

		private static const DIE_Y_UNSELECTED:int = 80;
		private static const DIE_Y_SELECTED:int = 10;
		
// ***** VARIABLES **********************************************************

		protected var _manager:IGameMode;
		protected var _setup:PlayerSetup;
		protected var _rollButton:TextButton;
		protected var _doneButton:TextButton;
		protected var _groupButton:TextButton;
		protected var _toTitle:TextButton;
		protected var _restartGame:TextButton;
		
		protected var _cars:Array;
		protected var _dice:Array;
		protected var _selected:Array;
		protected var _scores:Array;
		protected var _scoreText:Array;
		protected var _player:int;

		protected var _groupedDice:int;
		protected var _roundScore:int;
		protected var _sessionScore:int
		protected var _groupScore:int;
		
// ***** CONSTRUCTOR ********************************************************

		public function DixMilleGame(manager:IGameMode, setup:PlayerSetup)
		{
			var cntr:int, cntrPlayer:int;
			_manager = manager;
			_setup = setup;
			render();

			_rollButton = new TextButton(ROLL_TEXT, 200, 40);
			_rollButton.x = 10;
			_rollButton.y = 150;
			addChild(_rollButton);
			_rollButton.addEventListener(MouseEvent.MOUSE_UP, handleRoll);
			
			_doneButton = new TextButton(DONE_TEXT, 200, 40);
			_doneButton.x = 220;
			_doneButton.y = 150;
			addChild(_doneButton);
			_doneButton.addEventListener(MouseEvent.MOUSE_UP, handleDone);
			
			_groupButton = new TextButton(GROUP_TEXT, 200, 40);
			_groupButton.x = 430;
			_groupButton.y = 150;
			addChild(_groupButton);
			_groupButton.addEventListener(MouseEvent.MOUSE_UP, handleGroup);

			_dice = new Array(6);
			_cars = new Array(6);
			_selected = new Array(6);
			_scores = new Array(6);
			_scoreText = new Array(6);
			for (cntr = 0; cntr < 6; ++cntr) {
				_dice[cntr] = new GenericDie();
				_dice[cntr].x = 32 + 96 * cntr;
				_dice[cntr].y = DIE_Y_UNSELECTED;
				_dice[cntr].width = 64;
				_dice[cntr].height = 64;
				_dice[cntr].addEventListener(MouseEvent.MOUSE_UP, handleSelections);
				addChild(_dice[cntr]);
				_selected[cntr] = false;
				_cars[cntr] = new Car(PlayerSetup.PLAYER_COLORS[cntr],PlayerSetup.PLAYER_BACKGROUNDS[cntr]);
				_cars[cntr].x = 0;
				_cars[cntr].y = 205 + 40 * cntr;
				addChild(_cars[cntr]);
				_scoreText[cntr] = new TextField();
				_scoreText[cntr].x = 540;
				_scoreText[cntr].y = _cars[cntr].y;
				_scoreText[cntr].width = 100;
				_scoreText[cntr].height = 32;
				_scoreText[cntr].defaultTextFormat = TextButton.DEFAULT_FORMAT;
				_scoreText[cntr].textColor = 0xFFFFFF;
				_scoreText[cntr].text = "0";
				addChild(_scoreText[cntr]);
			}
			_toTitle = new TextButton("return to title...", 310, 20, SCORE_FORMAT);
			_toTitle.y = 460;
			addChild(_toTitle);
			_toTitle.addEventListener(MouseEvent.MOUSE_UP, handleToTitle);

			_restartGame = new TextButton("Start a new Round", 310, 20, SCORE_FORMAT);
			_restartGame.x = 330;
			_restartGame.y = 460;
			addChild(_restartGame);
			_restartGame.addEventListener(MouseEvent.MOUSE_UP, handleRestart);
			_restartGame.visible = false;
			startGame();
		}

// ***** Functions - Event Handling *****************************************

		private function handleSelections(evt:Event):void
		{
			var die:int = -1;
			var score:int = 0;
			var cntr:int;
			
			for (cntr = _groupedDice; cntr < _dice.length; ++cntr) {
				if (_dice[cntr] == evt.target)
					die = cntr;
			}
			if (die < 0) 
				return;
			_selected[die] = ! _selected[die];
			for (cntr = 0; cntr < _dice.length; ++cntr) {
				if (_selected[cntr]) {
					_dice[cntr].y = DIE_Y_SELECTED;
				} else {
					_dice[cntr].y = DIE_Y_UNSELECTED;
				}
			}
			// perform group check
			var groupArray:Array = new Array();
			for (cntr = _groupedDice; cntr < _dice.length; ++cntr) {
				if (_selected[cntr])
					groupArray.push(_dice[cntr].getValue());
			}
			// figure out score
			_groupScore = scoreGroup(groupArray);
			_groupButton.visible = (_groupScore > 0);
			
		}
		
		private function scoreGroup(groupArray:Array):int
		{
			var groupScore:int;

			groupArray.sort();
			if (groupArray.length == 6) {
				if ( (groupArray[0] == groupArray[1]) && (groupArray[2] == groupArray[3]) && (groupArray[4] == groupArray[5]) )
					groupScore = 1500;
				else {
					var isStraight:Boolean = true;
					for (var cntr:int = 1; cntr < 6; ++cntr) {
						if (groupArray[cntr-1] != (groupArray[cntr]-1))
							isStraight = false
					}
					if (isStraight)
						groupScore = 3000;
				}
			} else if (groupArray.length == 3) {
				if (groupArray[0] == groupArray[2]) {
					if (groupArray[0] == 1)
						groupScore = 1000;
					else
						groupScore = groupArray[0] * 100;
				}
			} else if (groupArray.length == 1) {
				if (groupArray[0] == 1)
					groupScore = 100;
				else if (groupArray[0] == 5)
					groupScore = 50;
			}
			return groupScore;
		}
		
		private function handleRoll(evt:Event):void
		{

			var cntr:int;
			
			if (_groupedDice == _dice.length)
				_groupedDice = 0;
			for (cntr = _groupedDice; cntr < _dice.length; ++cntr) {
				_dice[cntr].quickRoll();
				_selected[cntr] = false;
				_dice[cntr].y = DIE_Y_UNSELECTED;
				_dice[cntr].visible = true;
			}
			_rollButton.visible = false;
			_doneButton.setLabel("0");
			_roundScore = 0;
			_doneButton.visible = true;
		}

		private function handleDone(evt:Event):void
		{
			_scores[_player] += _roundScore;
			_scoreText[_player].text = ""+_scores[_player];
			_cars[_player].x = _scores[_player] / 20;
			
			nextPlayer();
		}

		private function handleGroup(evt:Event):void
		{
			var temp:int;
			
			for (var cntr:int = _groupedDice; cntr < _dice.length; ++cntr) {
				if (_selected[cntr]) {
					temp = _dice[_groupedDice].getValue();
					_dice[_groupedDice].setValue(_dice[cntr].getValue());
					_dice[cntr].setValue(temp);
					_selected[cntr] = false;
					_dice[cntr].y = DIE_Y_UNSELECTED;
					_selected[_groupedDice] = true
					_dice[_groupedDice].y = DIE_Y_SELECTED;
					++_groupedDice;
				}
			}
			_groupButton.visible = false;
			_sessionScore += _groupScore;
			_roundScore = _sessionScore;
			_groupScore = 0;
			_doneButton.setLabel(""+_roundScore);
			_doneButton.visible = true;
			_rollButton.visible = true;
			render();
		}
		
		public function handleToTitle(evt:MouseEvent):void
		{
			_manager.changeMode(1);
		}

		public function handleRestart(evt:MouseEvent):void
		{
			startGame();
		}

		
// ***** Functions - Rendering **********************************************

		private function render():void
		{
			graphics.clear();
			graphics.beginFill(0x808080);
            graphics.drawRect(0, 200, 640, 240);
            graphics.endFill();

			if (_player >= 0) {
				graphics.beginFill(0xC0C0C0);
				graphics.drawRect(0, 200 + 40* _player, 640, 40);
				graphics.endFill();
			}
			
			if (_groupedDice > 0) {
				graphics.beginFill(0x400040);
				graphics.drawRect(8, 0, _groupedDice * 96, 96);
				graphics.endFill();
			}
			
			graphics.lineStyle(3, 0xC0C000);
			var lineY:int;
			for (var cntr:int = 0; cntr < 8; ++cntr) {
				lineY = 200 + 40*cntr;
				graphics.moveTo(0,lineY);
				graphics.lineTo(640,lineY);
			}
			graphics.moveTo(32,200);
			graphics.lineTo(32,440);
			graphics.moveTo(530,200);
			graphics.lineTo(530,440);
			
//			graphics.endFill();

		}

// ***** Functions - Gameplay management ************************************

		public function startGame():void
		{
			var cntr:int;

			for (cntr = 0; cntr < _scores.length; ++cntr) {
				_scores[cntr] = 0;
				_cars[cntr].x = 0;
				_scoreText[cntr].text = "0";
			}
			_restartGame.visible = false;
			_player = -1;
			nextPlayer();
		}
		
		public function nextPlayer():void
		{
			var cntr:int;
			_rollButton.visible = true;
			_doneButton.visible = false;
			_groupButton.visible = false;
			_groupedDice = 0;
			_roundScore = 0;
			_sessionScore = 0;
			_groupScore = 0;
			
			for (cntr = 0; cntr < _dice.length; ++cntr)
				_selected[cntr] = false;
			
			++_player;
			if (_player > 5) {
				var highScore:int = 0;
				for (cntr = 0; cntr < _scores.length; ++cntr)
					if (_scores[cntr] > highScore)
						highScore = _scores[cntr];
				if (highScore >= 10000) {
					_rollButton.visible = false;
					_restartGame.visible = true;
					render();
					return;
				}
				_player = 0;
			}
			if (_setup.getPlayerType(_player) == PlayerSetup.PLAYER_COMPUTER){
				playAI();
			} else if (_setup.getPlayerType(_player) == PlayerSetup.PLAYER_NONE)
				nextPlayer();
				
			render();
		}
		
		private function playAI() : void
		{
			handleRoll(null);
			var temp:int;
			var best:int = 0;
			var bestFlags:int = 0;
			for (var cntr:int = 1; cntr < 64; ++cntr) {
				var groupArray:Array = new Array();
				if ((cntr & 1) == 1)
					groupArray.push(_dice[0].getValue());
				if ((cntr & 2) == 2)
					groupArray.push(_dice[1].getValue());
				if ((cntr & 4) == 4)
					groupArray.push(_dice[2].getValue());
				if ((cntr & 8) == 8)
					groupArray.push(_dice[3].getValue());
				if ((cntr & 16) == 16)
					groupArray.push(_dice[4].getValue());
				if ((cntr & 32) == 32)
					groupArray.push(_dice[5].getValue());
				temp = scoreGroup(groupArray);
				if (temp > best) {
					best = temp;
					bestFlags = cntr;
				}
			}
			if (best > 0) {
				_selected[0] = ((bestFlags & 1) == 1);
				_selected[1] = ((bestFlags & 2) == 2);
				_selected[2] = ((bestFlags & 4) == 4);
				_selected[3] = ((bestFlags & 8) == 8);
				_selected[4] = ((bestFlags & 16) == 16);
				_selected[5] = ((bestFlags & 32) == 32);
				_groupScore = best;
				handleGroup(null);
			}
			handleDone(null);
		}
		
	} // class
} // package
